@@ -37,132 +37,20 @@ class NetworkPolicyDriver(base.NetworkPolicyDriver):
     def __init__(self):
         self.os_net = clients.get_network_client()
         self.kubernetes = clients.get_kubernetes_client()
+        self.nodes_subnets_driver = base.NodesSubnetsDriver.get_instance()
 
-    def ensure_network_policy(self, policy):
-        """Create security group rules out of network policies
-
-        Triggered by events from network policies, this method ensures that
-        KuryrNetworkPolicy object is created with the security group rules
-        definitions required to represent the NetworkPolicy.
-        """
-        LOG.debug("Creating network policy %s", policy['metadata']['name'])
-
-        i_rules, e_rules = self._get_security_group_rules_from_network_policy(
-            policy)
-
-        knp = self._get_knp_crd(policy)
-        if not knp:
-            try:
-                self._create_knp_crd(policy, i_rules, e_rules)
-            except exceptions.K8sNamespaceTerminating:
-                LOG.warning('Namespace %s is being terminated, ignoring '
-                            'NetworkPolicy %s in that namespace.',
-                            policy['metadata']['namespace'],
-                            policy['metadata']['name'])
-                return
+    def affected_pods(self, policy, selector=None):
+        if selector is not None:
+            pod_selector = selector
         else:
-            self._patch_knp_crd(policy, i_rules, e_rules, knp)
-
-    def _convert_old_sg_rule(self, rule):
-        del rule['security_group_rule']['id']
-        del rule['security_group_rule']['security_group_id']
-        result = {
-            'sgRule': rule['security_group_rule'],
-        }
-
-        if 'namespace' in rule:
-            result['namespace'] = rule['namespace']
-
-        if 'remote_ip_prefixes' in rule:
-            result['affectedPods'] = []
-            for ip, namespace in rule['remote_ip_prefixes']:
-                if not ip:
-                    continue
-                result['affectedPods'].append({
-                    'podIP': ip,
-                    'podNamespace': namespace,
-                })
-
-        return result
-
-    def get_from_old_crd(self, netpolicy):
-        name = netpolicy['metadata']['name'][3:]  # Remove 'np-'
-        namespace = netpolicy['metadata']['namespace']
-        link = (f'{constants.K8S_API_NETWORKING}/namespaces/{namespace}/'
-                f'networkpolicies/{name}')
-        knp = {
-            'apiVersion': constants.K8S_API_CRD_VERSION,
-            'kind': constants.K8S_OBJ_KURYRNETWORKPOLICY,
-            'metadata': {
-                'namespace': namespace,
-                'name': name,
-                'annotations': {
-                    'networkPolicyLink': link,
-                },
-                'finalizers': [constants.NETWORKPOLICY_FINALIZER],
-            },
-            'spec': {
-                'podSelector':
-                    netpolicy['spec']['networkpolicy_spec']['podSelector'],
-                'egressSgRules': [self._convert_old_sg_rule(r) for r in
-                                  netpolicy['spec']['egressSgRules']],
-                'ingressSgRules': [self._convert_old_sg_rule(r) for r in
-                                   netpolicy['spec']['ingressSgRules']],
-                'policyTypes':
-                    netpolicy['spec']['networkpolicy_spec']['policyTypes'],
-            },
-            'status': {
-                'podSelector': netpolicy['spec']['podSelector'],
-                'securityGroupId': netpolicy['spec']['securityGroupId'],
-                # We'll just let KuryrNetworkPolicyHandler figure out if rules
-                # are created on its own.
-                'securityGroupRules': [],
-            },
-        }
-
-        return knp
-
-    def _get_security_group_rules_from_network_policy(self, policy):
-        """Get security group rules required to represent an NP
-
-        This method creates the security group rules bodies coming out of a
-        network policies' parsing.
-        """
-        i_rules, e_rules = self.parse_network_policy_rules(policy)
-        # Add default rules to allow traffic from host and svc subnet
-        i_rules += self._get_default_np_rules()
-
-        return i_rules, e_rules
-
-    def _get_default_np_rules(self):
-        """Add extra SG rule to allow traffic from svcs and host.
-
-        This method adds the base security group rules for the NP security
-        group:
-        - Ensure traffic is allowed from the services subnet
-        - Ensure traffic is allowed from the host
-        """
-        rules = []
-        default_cidrs = []
-        if CONF.octavia_defaults.enforce_sg_rules:
-            default_cidrs.append(utils.get_subnet_cidr(
-                CONF.neutron_defaults.service_subnet))
-        worker_subnet_id = CONF.pod_vif_nested.worker_nodes_subnet
-        if worker_subnet_id:
-            default_cidrs.append(utils.get_subnet_cidr(worker_subnet_id))
-        for cidr in default_cidrs:
-            ethertype = constants.IPv4
-            if ipaddress.ip_network(cidr).version == constants.IP_VERSION_6:
-                ethertype = constants.IPv6
-            rules.append({
-                'sgRule': {
-                    'ethertype': ethertype,
-                    'direction': 'ingress',
-                    'description': 'Kuryr-Kubernetes NetPolicy SG rule',
-                    'remote_ip_prefix': cidr,
-                }})
-
-        return rules
+            pod_selector = policy['spec'].get('podSelector')
+        if pod_selector:
+            policy_namespace = policy['metadata']['namespace']
+            pods = driver_utils.get_pods(pod_selector, policy_namespace)
+            return pods.get('items')
+        else:
+            # NOTE(ltomasbo): It affects all the pods on the namespace
+            return self.namespaced_pods(policy)
 
     def create_security_group(self, knp, project_id):
         sg_name = ("sg-" + knp['metadata']['namespace'] + "-" +
@@ -190,6 +78,92 @@ class NetworkPolicyDriver(base.NetworkPolicyDriver):
 
         return sg.id
 
+    def delete_np_sg(self, sg_id):
+        try:
+            self.os_net.delete_security_group(sg_id)
+        except os_exc.ConflictException:
+            LOG.debug("Security Group %s still in use!", sg_id)
+            # raising ResourceNotReady to retry this action in case ports
+            # associated to affected pods are not updated on time, i.e.,
+            # they are still using the security group to be removed
+            raise exceptions.ResourceNotReady(sg_id)
+        except os_exc.SDKException:
+            LOG.exception("Error deleting security group %s.", sg_id)
+            raise
+
+    def ensure_network_policy(self, policy):
+        """Create security group rules out of network policies
+
+        Triggered by events from network policies, this method ensures that
+        KuryrNetworkPolicy object is created with the security group rules
+        definitions required to represent the NetworkPolicy.
+        """
+        LOG.debug("Creating network policy %s", policy['metadata']['name'])
+
+        i_rules, e_rules = self._get_security_group_rules_from_network_policy(
+            policy)
+
+        knp = self._get_knp_crd(policy)
+        if not knp:
+            try:
+                self._create_knp_crd(policy, i_rules, e_rules)
+            except exceptions.K8sNamespaceTerminating:
+                LOG.warning('Namespace %s is being terminated, ignoring '
+                            'NetworkPolicy %s in that namespace.',
+                            policy['metadata']['namespace'],
+                            policy['metadata']['name'])
+                return
+        else:
+            self._patch_knp_crd(policy, i_rules, e_rules, knp)
+
+    def namespaced_pods(self, policy):
+        pod_namespace = policy['metadata']['namespace']
+        pods = self.kubernetes.get('{}/namespaces/{}/pods'.format(
+            constants.K8S_API_BASE, pod_namespace))
+        return pods.get('items')
+
+    def _get_security_group_rules_from_network_policy(self, policy):
+        """Get security group rules required to represent an NP
+
+        This method creates the security group rules bodies coming out of a
+        network policies' parsing.
+        """
+        i_rules, e_rules = self._parse_network_policy_rules(policy)
+        # Add default rules to allow traffic from host and svc subnet
+        i_rules += self._get_default_np_rules()
+
+        return i_rules, e_rules
+
+    def _get_default_np_rules(self):
+        """Add extra SG rule to allow traffic from svcs and host.
+
+        This method adds the base security group rules for the NP security
+        group:
+        - Ensure traffic is allowed from the services subnet
+        - Ensure traffic is allowed from the host
+        """
+        rules = []
+        default_cidrs = []
+        if CONF.octavia_defaults.enforce_sg_rules:
+            default_cidrs.append(utils.get_subnet_cidr(
+                CONF.neutron_defaults.service_subnet))
+        worker_subnet_ids = self.nodes_subnets_driver.get_nodes_subnets()
+        default_cidrs.extend(utils.get_subnets_cidrs(worker_subnet_ids))
+
+        for cidr in default_cidrs:
+            ethertype = constants.IPv4
+            if ipaddress.ip_network(cidr).version == constants.IP_VERSION_6:
+                ethertype = constants.IPv6
+            rules.append({
+                'sgRule': {
+                    'ethertype': ethertype,
+                    'direction': 'ingress',
+                    'description': 'Kuryr-Kubernetes NetPolicy SG rule',
+                    'remote_ip_prefix': cidr,
+                }})
+
+        return rules
+
     def _get_pods(self, pod_selector, namespace=None, namespace_selector=None):
         matching_pods = {"items": []}
         if namespace_selector:
@@ -214,7 +188,7 @@ class NetworkPolicyDriver(base.NetworkPolicyDriver):
 
     def _parse_selectors(self, rule_block, rule_direction, policy_namespace):
         allowed_resources = []
-        allow_all = False
+        allowed_cidrs = None
         selectors = False
         for rule in rule_block.get(rule_direction, []):
             namespace_selector = rule.get('namespaceSelector')
@@ -226,8 +200,13 @@ class NetworkPolicyDriver(base.NetworkPolicyDriver):
                     allowed_resources.extend(self._get_pods(
                         pod_selector))
                 else:
-                    # allow from all
-                    allow_all = True
+                    # allow from all the cluster, which means pod subnets and
+                    # service subnet.
+                    allowed_cidrs = utils.get_subnetpool_cidrs(
+                        CONF.namespace_subnet.pod_subnet_pool)
+                    if CONF.octavia_defaults.enforce_sg_rules:
+                        allowed_cidrs.append(utils.get_subnet_cidr(
+                            CONF.neutron_defaults.service_subnet))
             elif namespace_selector:
                 selectors = True
                 if pod_selector:
@@ -255,10 +234,10 @@ class NetworkPolicyDriver(base.NetworkPolicyDriver):
                         pod_selector,
                         namespace=policy_namespace))
 
-        return allow_all, selectors, allowed_resources
+        return allowed_cidrs, selectors, allowed_resources
 
     def _create_sg_rules_with_container_ports(
-        self, container_ports, allow_all, resource, matched_pods,
+        self, container_ports, allowed_cidrs, resource, matched_pods,
             crd_rules, direction, port, pod_selector=None,
             policy_namespace=None):
         cidr, ns = self._get_resource_details(resource)
@@ -279,11 +258,12 @@ class NetworkPolicyDriver(base.NetworkPolicyDriver):
                 matched_pods[container_port].update(pod_info)
             else:
                 matched_pods[container_port] = pod_info
-        if not allow_all and matched_pods and cidr:
+        if not allowed_cidrs and matched_pods and cidr:
             for container_port, pods in matched_pods.items():
                 sg_rule = driver_utils.create_security_group_rule_body(
                     direction, container_port,
-                    protocol=port.get('protocol'),
+                    # Pod's spec.containers[].port.protocol defaults to TCP
+                    protocol=port.get('protocol', 'TCP'),
                     cidr=cidr, pods=pods)
                 if sg_rule not in crd_rules:
                     crd_rules.append(sg_rule)
@@ -291,11 +271,13 @@ class NetworkPolicyDriver(base.NetworkPolicyDriver):
                     self._create_svc_egress_sg_rule(
                         policy_namespace, crd_rules,
                         resource=resource, port=container_port,
-                        protocol=port.get('protocol'))
+                        # Pod's spec.containers[].port.protocol defaults to TCP
+                        protocol=port.get('protocol', 'TCP'))
 
     def _create_sg_rule_body_on_text_port(self, direction, port,
                                           resources, crd_rules, pod_selector,
-                                          policy_namespace, allow_all=False):
+                                          policy_namespace,
+                                          allowed_cidrs=None):
         """Create SG rules when named port is used in the NP rule
 
         In case of ingress, the pods selected by NetworkPolicySpec's
@@ -314,8 +296,8 @@ class NetworkPolicyDriver(base.NetworkPolicyDriver):
         param crd_rules: list of parsed SG rules
         param pod_selector: dict with NetworkPolicySpec's podSelector
         param policy_namespace: string with policy namespace
-        param allow_all: True if should parse a allow from/to all rule,
-        False otherwise
+        param allowed_cidrs: None, or a list of cidrs, where/from the traffic
+                             should be allowed.
         """
         matched_pods = {}
         if direction == "ingress":
@@ -325,7 +307,7 @@ class NetworkPolicyDriver(base.NetworkPolicyDriver):
                 container_ports = driver_utils.get_ports(selected_pod, port)
                 for resource in resources:
                     self._create_sg_rules_with_container_ports(
-                        container_ports, allow_all, resource, matched_pods,
+                        container_ports, allowed_cidrs, resource, matched_pods,
                         crd_rules, direction, port)
         elif direction == "egress":
             for resource in resources:
@@ -337,23 +319,19 @@ class NetworkPolicyDriver(base.NetworkPolicyDriver):
                     continue
                 container_ports = driver_utils.get_ports(resource, port)
                 self._create_sg_rules_with_container_ports(
-                    container_ports, allow_all, resource, matched_pods,
+                    container_ports, allowed_cidrs, resource, matched_pods,
                     crd_rules, direction, port, pod_selector,
                     policy_namespace)
-        if allow_all:
-            container_port = None
+        if allowed_cidrs:
             for container_port, pods in matched_pods.items():
-                for ethertype in (constants.IPv4, constants.IPv6):
+                for cidr in allowed_cidrs:
                     sg_rule = driver_utils.create_security_group_rule_body(
                         direction, container_port,
-                        protocol=port.get('protocol'),
-                        ethertype=ethertype,
+                        # Pod's spec.containers[].port.protocol defaults to TCP
+                        protocol=port.get('protocol', 'TCP'),
+                        cidr=cidr,
                         pods=pods)
                     crd_rules.append(sg_rule)
-            if direction == 'egress':
-                self._create_svc_egress_sg_rule(
-                    policy_namespace, crd_rules,
-                    port=container_port, protocol=port.get('protocol'))
 
     def _create_sg_rule_on_number_port(self, allowed_resources,
                                        direction, port, sg_rule_body_list,
@@ -368,7 +346,8 @@ class NetworkPolicyDriver(base.NetworkPolicyDriver):
             sg_rule = (
                 driver_utils.create_security_group_rule_body(
                     direction, port.get('port'),
-                    protocol=port.get('protocol'),
+                    # NP's ports[].protocol defaults to TCP
+                    protocol=port.get('protocol', 'TCP'),
                     cidr=cidr,
                     namespace=ns))
             sg_rule_body_list.append(sg_rule)
@@ -376,30 +355,34 @@ class NetworkPolicyDriver(base.NetworkPolicyDriver):
                 self._create_svc_egress_sg_rule(
                     policy_namespace, sg_rule_body_list,
                     resource=resource, port=port.get('port'),
-                    protocol=port.get('protocol'))
+                    # NP's ports[].protocol defaults to TCP
+                    protocol=port.get('protocol', 'TCP'))
 
     def _create_all_pods_sg_rules(self, port, direction,
                                   sg_rule_body_list, pod_selector,
-                                  policy_namespace):
-        if type(port.get('port')) is not int:
+                                  policy_namespace, allowed_cidrs=None):
+        if not isinstance(port.get('port'), int):
             all_pods = driver_utils.get_namespaced_pods().get('items')
             self._create_sg_rule_body_on_text_port(
                 direction, port, all_pods,
                 sg_rule_body_list, pod_selector, policy_namespace,
-                allow_all=True)
+                allowed_cidrs=allowed_cidrs)
+        elif allowed_cidrs:
+            for cidr in allowed_cidrs:
+                sg_rule = driver_utils.create_security_group_rule_body(
+                        direction, port.get('port'),
+                        protocol=port.get('protocol'),
+                        cidr=cidr)
+                sg_rule_body_list.append(sg_rule)
         else:
             for ethertype in (constants.IPv4, constants.IPv6):
                 sg_rule = (
                     driver_utils.create_security_group_rule_body(
                         direction, port.get('port'),
                         ethertype=ethertype,
-                        protocol=port.get('protocol')))
+                        # NP's ports[].protocol defaults to TCP
+                        protocol=port.get('protocol', 'TCP')))
                 sg_rule_body_list.append(sg_rule)
-                if direction == 'egress':
-                    self._create_svc_egress_sg_rule(
-                        policy_namespace, sg_rule_body_list,
-                        port=port.get('port'),
-                        protocol=port.get('protocol'))
 
     def _create_default_sg_rule(self, direction, sg_rule_body_list):
         for ethertype in (constants.IPv4, constants.IPv6):
@@ -422,9 +405,11 @@ class NetworkPolicyDriver(base.NetworkPolicyDriver):
         It accounts for special cases, such as:
         - PolicyTypes stating only Egress: ensuring ingress is not restricted
         - PolicyTypes not including Egress: ensuring egress is not restricted
-        - {} ingress/egress rules: applying default open for all
+        - {} ingress/egress rules: applying default open for all the cluster
         """
+        _create_sg_rule_body = driver_utils.create_security_group_rule_body
         rule_list = policy['spec'].get(direction)
+
         if not rule_list:
             policy_types = policy['spec'].get('policyTypes')
             if direction == 'ingress':
@@ -432,18 +417,18 @@ class NetworkPolicyDriver(base.NetworkPolicyDriver):
                     # NOTE(ltomasbo): add default rule to enable all ingress
                     # traffic as NP policy is not affecting ingress
                     LOG.debug('Applying default all open for ingress for '
-                              'policy %s', policy['metadata']['selfLink'])
+                              'policy %s', utils.get_res_link(policy))
                     self._create_default_sg_rule(direction, sg_rule_body_list)
             elif direction == 'egress':
                 if policy_types and 'Egress' not in policy_types:
                     # NOTE(ltomasbo): add default rule to enable all egress
                     # traffic as NP policy is not affecting egress
                     LOG.debug('Applying default all open for egress for '
-                              'policy %s', policy['metadata']['selfLink'])
+                              'policy %s', utils.get_res_link(policy))
                     self._create_default_sg_rule(direction, sg_rule_body_list)
             else:
                 LOG.warning('Not supported policyType at network policy %s',
-                            policy['metadata']['selfLink'])
+                            utils.get_res_link(policy))
             return
 
         policy_namespace = policy['metadata']['namespace']
@@ -455,17 +440,18 @@ class NetworkPolicyDriver(base.NetworkPolicyDriver):
 
         if rule_list[0] == {}:
             LOG.debug('Applying default all open policy from %s',
-                      policy['metadata']['selfLink'])
+                      utils.get_res_link(policy))
             for ethertype in (constants.IPv4, constants.IPv6):
-                rule = driver_utils.create_security_group_rule_body(
-                    direction, ethertype=ethertype)
+                rule = _create_sg_rule_body(direction, ethertype=ethertype)
                 sg_rule_body_list.append(rule)
 
         for rule_block in rule_list:
             LOG.debug('Parsing %(dir)s Rule %(rule)s', {'dir': direction,
                                                         'rule': rule_block})
-            allow_all, selectors, allowed_resources = self._parse_selectors(
-                rule_block, rule_direction, policy_namespace)
+            (allowed_cidrs, selectors,
+             allowed_resources) = self._parse_selectors(rule_block,
+                                                        rule_direction,
+                                                        policy_namespace)
 
             ipblock_list = []
 
@@ -487,8 +473,8 @@ class NetworkPolicyDriver(base.NetworkPolicyDriver):
 
             if 'ports' in rule_block:
                 for port in rule_block['ports']:
-                    if allowed_resources or allow_all or selectors:
-                        if type(port.get('port')) is not int:
+                    if allowed_resources or allowed_cidrs or selectors:
+                        if not isinstance(port.get('port'), int):
                             self._create_sg_rule_body_on_text_port(
                                 direction, port, allowed_resources,
                                 sg_rule_body_list, pod_selector,
@@ -497,15 +483,15 @@ class NetworkPolicyDriver(base.NetworkPolicyDriver):
                             self._create_sg_rule_on_number_port(
                                 allowed_resources, direction, port,
                                 sg_rule_body_list, policy_namespace)
-                        if allow_all:
+                        if allowed_cidrs:
                             self._create_all_pods_sg_rules(
                                 port, direction, sg_rule_body_list,
-                                pod_selector, policy_namespace)
+                                pod_selector, policy_namespace, allowed_cidrs)
                     else:
                         self._create_all_pods_sg_rules(
                             port, direction, sg_rule_body_list,
                             pod_selector, policy_namespace)
-            elif allowed_resources or allow_all or selectors:
+            elif allowed_resources or allowed_cidrs or selectors:
                 for resource in allowed_resources:
                     cidr, namespace = self._get_resource_details(resource)
                     # NOTE(maysams): Skipping resource that do not have
@@ -513,34 +499,23 @@ class NetworkPolicyDriver(base.NetworkPolicyDriver):
                     # will be triggered again after the resource is running.
                     if not cidr:
                         continue
-                    rule = driver_utils.create_security_group_rule_body(
-                        direction,
-                        port_range_min=1,
-                        port_range_max=65535,
-                        cidr=cidr,
-                        namespace=namespace)
+                    rule = _create_sg_rule_body(direction, cidr=cidr,
+                                                namespace=namespace)
                     sg_rule_body_list.append(rule)
                     if direction == 'egress':
                         self._create_svc_egress_sg_rule(
                             policy_namespace, sg_rule_body_list,
                             resource=resource)
-                if allow_all:
-                    for ethertype in (constants.IPv4, constants.IPv6):
-                        rule = driver_utils.create_security_group_rule_body(
-                            direction,
-                            port_range_min=1,
-                            port_range_max=65535,
-                            ethertype=ethertype)
+                if allowed_cidrs:
+                    for cidr in allowed_cidrs:
+                        rule = _create_sg_rule_body(direction, cidr=cidr)
                         sg_rule_body_list.append(rule)
-                        if direction == 'egress':
-                            self._create_svc_egress_sg_rule(policy_namespace,
-                                                            sg_rule_body_list)
             else:
                 LOG.debug('This network policy specifies no %(direction)s '
                           '%(rule_direction)s and no ports: %(policy)s',
                           {'direction': direction,
                            'rule_direction': rule_direction,
-                           'policy': policy['metadata']['selfLink']})
+                           'policy': utils.get_res_link(policy)})
 
     def _create_svc_egress_sg_rule(self, policy_namespace, sg_rule_body_list,
                                    resource=None, port=None, protocol=None):
@@ -555,32 +530,46 @@ class NetworkPolicyDriver(base.NetworkPolicyDriver):
             return
 
         for service in services.get('items'):
+            svc_name = service['metadata']['name']
+            svc_namespace = service['metadata']['namespace']
             if self._is_pod(resource):
                 pod_labels = resource['metadata'].get('labels')
                 svc_selector = service['spec'].get('selector')
-                if not svc_selector or not pod_labels:
-                    continue
-                else:
+                if not svc_selector:
+                    targets = driver_utils.get_endpoints_targets(
+                            svc_name, svc_namespace)
+                    pod_ip = resource['status'].get('podIP')
+                    if pod_ip and pod_ip not in targets:
+                        continue
+                elif pod_labels:
                     if not driver_utils.match_labels(
                             svc_selector, pod_labels):
                         continue
             elif resource.get('cidr'):
                 # NOTE(maysams) Accounts for traffic to pods under
                 # a service matching an IPBlock rule.
-                svc_namespace = service['metadata']['namespace']
-                if svc_namespace != policy_namespace:
-                    continue
                 svc_selector = service['spec'].get('selector')
-                pods = driver_utils.get_pods({'selector': svc_selector},
-                                             svc_namespace).get('items')
-                if not self._pods_in_ip_block(pods, resource):
-                    continue
+                if not svc_selector:
+                    # Retrieving targets of services on any Namespace
+                    targets = driver_utils.get_endpoints_targets(
+                        svc_name, svc_namespace)
+                    if (not targets or
+                            not self._targets_in_ip_block(targets, resource)):
+                        continue
+                else:
+                    if svc_namespace != policy_namespace:
+                        continue
+                    pods = driver_utils.get_pods({'selector': svc_selector},
+                                                 svc_namespace).get('items')
+                    if not self._pods_in_ip_block(pods, resource):
+                        continue
             else:
                 ns_name = service['metadata']['namespace']
                 if ns_name != resource['metadata']['name']:
                     continue
             cluster_ip = service['spec'].get('clusterIP')
-            if not cluster_ip:
+            if not cluster_ip or cluster_ip == 'None':
+                # Headless services has 'None' as clusterIP.
                 continue
             rule = driver_utils.create_security_group_rule_body(
                 'egress', port, protocol=protocol,
@@ -596,7 +585,14 @@ class NetworkPolicyDriver(base.NetworkPolicyDriver):
                 return True
         return False
 
-    def parse_network_policy_rules(self, policy):
+    def _targets_in_ip_block(self, targets, resource):
+        for target in targets:
+            if (ipaddress.ip_address(target)
+                    not in ipaddress.ip_network(resource.get('cidr'))):
+                return False
+        return True
+
+    def _parse_network_policy_rules(self, policy):
         """Create security group rule bodies out of network policies.
 
         Whenever a notification from the handler 'on-present' method is
@@ -612,19 +608,6 @@ class NetworkPolicyDriver(base.NetworkPolicyDriver):
 
         return ingress_sg_rule_body_list, egress_sg_rule_body_list
 
-    def delete_np_sg(self, sg_id):
-        try:
-            self.os_net.delete_security_group(sg_id)
-        except os_exc.ConflictException:
-            LOG.debug("Security Group %s still in use!", sg_id)
-            # raising ResourceNotReady to retry this action in case ports
-            # associated to affected pods are not updated on time, i.e.,
-            # they are still using the security group to be removed
-            raise exceptions.ResourceNotReady(sg_id)
-        except os_exc.SDKException:
-            LOG.exception("Error deleting security group %s.", sg_id)
-            raise
-
     def release_network_policy(self, policy):
         return self._del_knp_crd(policy)
 
@@ -655,7 +638,7 @@ class NetworkPolicyDriver(base.NetworkPolicyDriver):
                 'name': networkpolicy_name,
                 'namespace': namespace,
                 'annotations': {
-                    'networkPolicyLink': policy['metadata']['selfLink'],
+                    'networkPolicyLink': utils.get_res_link(policy)
                 },
                 'finalizers': [constants.NETWORKPOLICY_FINALIZER],
             },
@@ -718,25 +701,6 @@ class NetworkPolicyDriver(base.NetworkPolicyDriver):
                           "KuryrNetworkPolicy CRD %s." % name)
             raise
 
-    def affected_pods(self, policy, selector=None):
-        if selector is not None:
-            pod_selector = selector
-        else:
-            pod_selector = policy['spec'].get('podSelector')
-        if pod_selector:
-            policy_namespace = policy['metadata']['namespace']
-            pods = driver_utils.get_pods(pod_selector, policy_namespace)
-            return pods.get('items')
-        else:
-            # NOTE(ltomasbo): It affects all the pods on the namespace
-            return self.namespaced_pods(policy)
-
-    def namespaced_pods(self, policy):
-        pod_namespace = policy['metadata']['namespace']
-        pods = self.kubernetes.get('{}/namespaces/{}/pods'.format(
-            constants.K8S_API_BASE, pod_namespace))
-        return pods.get('items')
-
     def _get_resource_details(self, resource):
         namespace = None
         if self._is_pod(resource):
